#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing,
#  software distributed under the License is distributed on an
#  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#  KIND, either express or implied.  See the License for the
#  specific language governing permissions and limitations
#  under the License.

from enum import Enum

import attr

from sedona.core.jvm.abstract import JvmObject
from sedona.utils.decorators import require


class IndexType(Enum):
    QUADTREE = "QUADTREE"
    RTREE = "RTREE"

    @classmethod
    def from_string(cls, index: str):
        try:
            index_type = getattr(cls, index)
        except AttributeError:
            raise AttributeError(f"Can not found {index}")
        return index_type


@attr.s
class IndexTypeJvm(JvmObject):
    index_type = attr.ib(type=IndexType)

    def _create_jvm_instance(self):
        return self.jvm_index(self.index_type.value) if self.index_type.value is not None else None

    @property
    @require(["FileDataSplitter"])
    def jvm_index(self):
        return self.jvm.IndexType.getIndexType
